﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcNoe.DAL;

namespace MvcNoe.Controllers
{
    public class ForumMissatgeController : Controller
    {
        private SONINEntities db = new SONINEntities();


        //
        // GET: /ForumMissatge/

        public ViewResult Index(string Temes)
        {
            try
            {
                //The first few lines of code create a List object to hold temes from the database. 
                var TemesLst = new List<string>();

                var TemesQry = from t in db.TBL_Temes
                               orderby t.desc_tema
                               select t.id_tema;

                TemesLst.AddRange(TemesQry);
                ViewBag.Temes = new SelectList(TemesLst);


                //filtrem els missatges que mostrem per tema
                var missatges = from m in db.SP_IndexForumMissatge_EXTEND()
                                select m;

                //The following code shows how to check the parameter. 
                //If it's not empty the code further constrains the query to limit
                //the selected to the specified .
                if (string.IsNullOrEmpty(Temes))
                {
                   
                    return View(missatges.ToList());
                }
                else
                {
                    
                    return View(missatges.ToList().Where(x => x.id_tema.Trim() == Temes.Trim()));
                }

                //filtrem només els actius. cridem un stor.proc
                //return View(db.SP_IndexForumMissatge_EXTEND().ToList());
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return View("Error");
            }
        }

        /// <summary>
        /// Initialization method for the SelectDiscussionControl. 
        /// Allows running server-side code before displaying partial view.
        /// El cridem al inici de la pàgina Index.aspx
        /// </summary>
        /// <returns>PartialView</returns>
        public ActionResult InitializeUserControl(string Temes)
        {
            //Notice in the initialization method, after setting up the model, 
            //we simply return the PartialView itself and pass in the required model. 
            //In this way, the rendering partial view will utilize the model and display the contents appropriately. 
            return PartialView("UserControls/SelectDiscussionControl", db.SP_Select_Discussions());
        }

        //
        // GET: /ForumMissatge/Details/5

        public ViewResult Details(string id)
        {
            try
            {
                return View(getMissatge(id));
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return View("Error");
            }
        }

        //
        // GET: /ForumMissatge/Edit/5

        public ActionResult Edit(string id)
        {
            try
            {
                return View(getMissatge(id));
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return RedirectToAction("Error");
            }
        }

        //
        // POST: /ForumMissatge/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_Forums_missatges tbl_forums_missatges)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Forums_missatges.Attach(tbl_forums_missatges);
                    db.ObjectStateManager.ChangeObjectState(tbl_forums_missatges, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_forums_missatges);
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return RedirectToAction("Error");
            }
        }


        // Tornem un llistat dels temes, forums i discussions
        // GET: /ForumMissatge/SelectDiscussion

        //public ViewResult SelectDiscussion()
        //{
        //    try
        //    {
                
        //        //return View(db.TBL_Forums_discussions.ToList());
        //        //fem servir una crida a un stored procedure
        //        return View(db.SP_Select_Discussions().ToList());
                     
        //    }
        //    catch (Exception e)
        //    {
        //        //passem el missatge de error a la vista
        //        ViewBag.error = e.Message;
        //        //adrecem la vista de errors
        //        return View("Error");
        //    }
        //}

        //Escriure un missatge a la discussió del fòrum del tema que tenim 
        //al id
        // GET: /ForumMissatge/Missatge

        public ActionResult Missatge(String id)
        {
            try
            {
                //la cadena és del tipus a_b_c_d
                //volem una matriu amb a b c d
                string[] a = id.Split('_');

                //asociem els valors a unes variables més descriptives
                ViewBag.id_tema = a[0];
                ViewBag.id_forum = a[1];
                ViewBag.id_forum_discussio = a[2];
                //data i hora sense / i sense : com a valor del id 
                ViewBag.id_forum_missatge = System.DateTime.Now.ToString().Replace("/", "").Replace(":", "").Replace(" ", "");

                //si tenim el usuari el passem a la vista
                if (Session["usuari"] != null)
                {
                    ViewBag.id_usuari_alta = (string)Session["usuari"];
                }
                return View();
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return RedirectToAction("Error");
            }
        }

        //
        // POST: /ForumMissatge/Missatge/5

        [HttpPost]
        public ActionResult Missatge(TBL_Forums_missatges tbl_forums_missatges)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Forums_missatges.AddObject(tbl_forums_missatges);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                return View(tbl_forums_missatges);
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //tornem la vista de Errors
                return View("Error");
            }
        }

        //Action per a cridar a la vista de error
        // POST: /ForumMissatge/Error

        public ViewResult Error()
        {
            return View();
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        /// <summary>
        /// retornem un objecte del tipus Forum Missatge
        /// </summary>
        /// <param name="id">cadena que rebem</param>
        /// <returns> retornem un objecte del tipus Forum Missatge que correspon als paràmetres que rebem a la cadena id</returns>
        private TBL_Forums_missatges getMissatge(string id)
        {
            //tractem la cadena id que rebem
            string idTema = "";
            string idForum = "";
            string idForumDiscussio = "";
            string idMissatge = "";

            //la cadena és del tipus a_b_c_d
            //volem una matriu amb a b c d
            string[] a = id.Split('_');

            //asociem els valors a unes variables més descriptives
            idTema = a[0];
            idForum = a[1];
            idForumDiscussio = a[2];
            idMissatge = a[3];

            TBL_Forums_missatges tbl_forums_missatges = db.TBL_Forums_missatges.Single(t => t.id_tema == idTema && t.id_forum == idForum && t.id_forum_discussio == idForumDiscussio && t.id_forum_missatge == idMissatge);
            return tbl_forums_missatges;
        }

    }
}