﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcSonin.DAL;
//per a exportar a excel
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

namespace MvcSonin.Controllers
{ 
    public class ForumMissatgeController : Controller
    {
        private SONINEntities db = new SONINEntities();

        //
        // GET: /ForumMissatge/

        public ViewResult Index()
        {
            try
            {
                //missatges actius
                //return View(db.TBL_Forums_missatges.ToList().Where(t => t.data_baixa == null));
                return View(db.SP_IndexForumMissatge_EXTEND().ToList());
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // GET: /ForumMissatge/Details/5

        public ViewResult Details(string id)
        {
            try
            {
                return View(getMissatge(id));
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

 
        public ActionResult Edit(string id)
        {
            try
            {
                return View(getMissatge(id));
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // POST: /ForumMissatge/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_Forums_missatges tbl_forums_missatges)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Forums_missatges.Attach(tbl_forums_missatges);
                    db.ObjectStateManager.ChangeObjectState(tbl_forums_missatges, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_forums_missatges);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // GET: /ForumMissatge/Delete/5
 
        public ActionResult Delete(string id)
        {
            try
            {
                return View(getMissatge(id));
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        // No esborrem el registre. Editem el usuari i la data de baixa
        // POST: /ForumMissatge/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(string id)
        {            
            try
            {

                TBL_Forums_missatges tbl_forums_missatges = getMissatge(id);

                //editem el usuari i la data de baixa
                tbl_forums_missatges.id_usuari_baixa = Session["usuari"].ToString();
                tbl_forums_missatges.data_baixa = System.DateTime.Now;

                if (ModelState.IsValid)
                {
                    db.ObjectStateManager.ChangeObjectState(tbl_forums_missatges, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_forums_missatges);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }



        }


        /// <summary>
        /// Exportem a excel
        /// </summary>
        /// <returns></returns>
        public ActionResult toExcel()
        {
            try
            {

                exportToExcel();
                return View();
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// métode per a exportar a Excel
        /// </summary>
        public void exportToExcel()
        {
            //This one is pretty easy, consisting of three parts:

            // * Creating the GridView placeholder
            // * Binding the query results to the GridView placeholder
            // * Dumping the results back to the browser

            // This example queries all contacts that have not unsubscribed, but you can imagine how easy it is to extend.
            var datos = db.SP_IndexForumMissatge_EXTEND().ToList();
            var grid = new System.Web.UI.WebControls.GridView();

            grid.DataSource = from container in datos

                              select new
                              {
                                  Tema = container.desc_tema,
                                  Forum = container.desc_forum,
                                  Nom_Discussio = container.titol,
                                  Missatge = container.missatge,
                                  Id_usuari_alta = container.id_usuari_alta,
                                  Data_alta = container.data_alta
                              };
            grid.DataBind();

            Response.ClearContent();
            Response.AddHeader("content-disposition", "attachment; filename=sonin.xls");
            Response.ContentType = "application/excel";
            StringWriter sw = new StringWriter();
            HtmlTextWriter htw = new HtmlTextWriter(sw);
            grid.RenderControl(htw);
            Response.Write(sw.ToString());
            Response.End();
        }

        // Tornem un llistat dels temes, forums i discussions
        // GET: /ForumMissatge/SelectDiscussion

        public ViewResult SelectDiscussion()
        {
            try
            {
                //discussions actives
                return View(db.SP_Select_Discussions().ToList());
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return View("Error");
            }
        }

        //Escriure un missatge a la discussió del fòrum del tema que tenim 
        //al id
        // GET: /ForumMissatge/Missatge

        public ActionResult Missatge(String id)
        {
            try
            {
                //la cadena és del tipus a_b_c_d
                //volem una matriu amb a b c d
                string[] a = id.Split('_');

                //asociem els valors a unes variables més descriptives
                ViewBag.id_tema = a[0];
                ViewBag.id_forum = a[1];
                ViewBag.id_forum_discussio = a[2];
                //data i hora sense / i sense : com a valor del id 
                ViewBag.id_forum_missatge = "Miss" + System.DateTime.Now.ToString().Replace("/", "").Replace(":", "").Replace(" ", "");
                ViewBag.id_usuari_alta =Session["usuari"];

                return View();
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //adrecem la vista de errors
                return RedirectToAction("Error");
            }
        }

        //
        // POST: /ForumMissatge/Missatge/5

        [HttpPost]
        public ActionResult Missatge(TBL_Forums_missatges tbl_forums_missatges)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Forums_missatges.AddObject(tbl_forums_missatges);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                return View(tbl_forums_missatges);
            }
            catch (Exception e)
            {
                //passem el missatge de error a la vista
                ViewBag.error = e.Message;
                //tornem la vista sense camps. 
                return View();
            }
        }

        // Cridem la vista de error
        // GET: /ForumMissatge/Error

        public ViewResult Error()
        {
            return View();
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }

        /// <summary>
        /// retornem un objecte del tipus Forum Missatge
        /// </summary>
        /// <param name="id">cadena que rebem</param>
        /// <returns> retornem un objecte del tipus Forum Missatge que correspon als paràmetres que rebem a la cadena id</returns>
        private TBL_Forums_missatges getMissatge(string id)
        {
            //tractem la cadena id que rebem
            string idTema = "";
            string idForum = "";
            string idForumDiscussio = "";
            string idMissatge = "";

            //la cadena és del tipus a_b_c_d
            //volem una matriu amb a b c d
            string[] a = id.Split('_');

            //asociem els valors a unes variables més descriptives
            idTema = a[0];
            idForum = a[1];
            idForumDiscussio = a[2];
            idMissatge = a[3];

            TBL_Forums_missatges tbl_forums_missatges = db.TBL_Forums_missatges.Single(t => t.id_tema == idTema && t.id_forum == idForum && t.id_forum_discussio == idForumDiscussio && t.id_forum_missatge == idMissatge);
            return tbl_forums_missatges;
        }
    }
}