﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcSonin.DAL;
//per a exportar a excel
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;


namespace MvcSonin.Controllers
{ 
    public class SAPController : Controller
    {
        private SONINEntities db = new SONINEntities();

        //
        // GET: /SAP/

        public ViewResult Index()
        {
             return View(db.TBL_Saps.ToList());
        }

        //
        // GET: /SAP/Details/5
        
        public ViewResult Details(string id)
        {
            try
            {
                TBL_Saps tbl_saps = db.TBL_Saps.Single(t => t.id_sap == id);
                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // GET: /SAP/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /SAP/Create
        [HttpPost]
        public ActionResult Create(TBL_Saps tbl_saps)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Saps.AddObject(tbl_saps);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View(tbl_saps);  
            }
        }


        //
        // GET: /SAP/Edit/5
 
        public ActionResult Edit(string id)
        {
            try
            {
                TBL_Saps tbl_saps = db.TBL_Saps.Single(t => t.id_sap == id);
                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /SAP/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_Saps tbl_saps)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Saps.Attach(tbl_saps);
                    db.ObjectStateManager.ChangeObjectState(tbl_saps, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View(tbl_saps);
            }
        }

        //
        // GET: /SAP/Delete/5
 
        public ActionResult Delete(string id)
        {
            try
            {
                TBL_Saps tbl_saps = db.TBL_Saps.Single(t => t.id_sap == id);
                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /SAP/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(string id)
        {
            try
            {
                //Al esborrar no esborrem sinò que afegim la data de baixa
                TBL_Saps tbl_saps = db.TBL_Saps.Single(t => t.id_sap == id);

                if (ModelState.IsValid)
                {
                    tbl_saps.data_baixa = System.DateTime.Now; //la data d'avui
         
                    db.ObjectStateManager.ChangeObjectState(tbl_saps, EntityState.Modified);
                    db.SaveChanges();

                    //donem de baixa els objectes lligats amb la sap
                    UpdateFK(tbl_saps.id_sap, Session["usuari"].ToString());

                    return RedirectToAction("Index");
                }
                return View(tbl_saps);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }

        }

        /// <summary>
        /// Exportem a excel
        /// </summary>
        /// <returns></returns>
        public ActionResult toExcel()
        {
            try
            {

                exportToExcel();
                return View();
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// métode per a exportar a Excel
        /// </summary>
        public void exportToExcel()
        {
            //This one is pretty easy, consisting of three parts:

            // * Creating the GridView placeholder
            // * Binding the query results to the GridView placeholder
            // * Dumping the results back to the browser

            // This example queries all contacts that have not unsubscribed, but you can imagine how easy it is to extend.
            var sapes = db.TBL_Saps.ToList();
            var grid = new System.Web.UI.WebControls.GridView();

            grid.DataSource = from saps in sapes

                              select new
                              {
                                  Id_sap = saps.id_sap,                
                                  Nom_sap = saps.desc_sap,
                                  Data_alta = saps.data_alta
                              };
            grid.DataBind();

            Response.ClearContent();
            Response.AddHeader("content-disposition", "attachment; filename=sonin.xls");
            Response.ContentType = "application/excel";
            StringWriter sw = new StringWriter();
            HtmlTextWriter htw = new HtmlTextWriter(sw);
            grid.RenderControl(htw);
            Response.Write(sw.ToString());
            Response.End();
        }


        /// <summary>
        /// Actualitzem les dates de baixa i el usuari de baixa
        /// dels objectes relacionats amb la SAP
        /// </summary>
        /// <param name="idSAP">id de la SAP</param>
        /// <param name="idUsuariBaixa">id del  usuari de baixa</param>
        private void UpdateFK(String idSAP, String idUsuariBaixa)
        {

            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand();
            System.Data.SqlClient.SqlConnection cn = new System.Data.SqlClient.SqlConnection();

            //la cadena de connexió
            cn.ConnectionString = "Server = (local); Database = sonin; User ID = sonin; Password = sonin; Trusted_Connection = False";

            // si la connexió està tancada la obrim.
            if (cn.State == ConnectionState.Closed)
            {
                cn.Open();
            }

            cmd.Connection = cn;
            //Les UP
            cmd.CommandText = "UPDATE TBL_UP SET data_baixa = '" + System.DateTime.Now.ToString() + "' WHERE codi_sap = '" + idSAP + "'";
            cmd.ExecuteNonQuery();

            //els centres
            cmd.CommandText = "UPDATE TBL_Centres SET data_baixa = '" + System.DateTime.Now.ToString() + "' WHERE codi_sap = '" + idSAP + "'";
            cmd.ExecuteNonQuery();

            //tanquem la connexió oberta
            if (cn.State == ConnectionState.Open)
            {
                cn.Close();
            }

        }


        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}