﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcSonin.DAL;
//per a exportar a excel
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

namespace MvcSonin.Controllers
{ 
    public class TemaController : Controller
    {
        private SONINEntities db = new SONINEntities();

        //
        // GET: /Tema/

        public ViewResult Index()
        {
            // return View(db.TBL_Temes.ToList());
            //fem servir la crida una fució que crida a un stored proc.
            return View(db.SP_IndexTema());
           
        }

        //
        // GET: /Tema/Details/5

        public ViewResult Details(string id)
        {
            try
            {
                //fent servir LINQ
                //TBL_Temes tbl_temes = db.TBL_Temes.Single(t => t.id_tema == id);
                //return View(tbl_temes);

                //fem servir una crida una funció que crida a un stored proced.
                //com que ens tornen un conjunt de resutats i només volem un (i més a més els tipus no coincidexien) hem fet el "apaño" de fer servir ElementAt(0).
                //Només hi ha un sol element. Millorar el procés a posteriori
                TBL_Temes tbl_temes = (TBL_Temes)db.SP_DetailsTema(id).ElementAt(0);
                return View(tbl_temes);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // GET: /Tema/Create

        public ActionResult Create()
        {
            //afegim els valors que usarem en els desplegables dels usuaris d'alta i de baixa en el Viewbag.
            //ViewBag.id_usuari_alta = new SelectList(db.TBL_Usuaris, "id_usuari","nom-usuari");
            return View(); 
        } 

        //
        // POST: /Tema/Create

        [HttpPost]
        public ActionResult Create(TBL_Temes tbl_temes)
        {
            try
            {
                //insertem els valors de manera automàtica
                tbl_temes.id_usuari_alta = Session["usuari"].ToString();
                tbl_temes.id_tema = "Tema" + System.DateTime.Now.ToString().Replace("/", "").Replace(":", "").Replace(" ", "");

                if (ModelState.IsValid)
                {
                    db.TBL_Temes.AddObject(tbl_temes);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                //afegim per a que surti el valor escollit als desplegables
                //VERY IMPORTANT
                //el nom id_usuari_alta del ViewBag ha de ser igual que el nom del camp de la taula. Sinò
                //quan es genera el desplegable a la vista el nom per a després desar les dades ha de ésser
                //el mateix.
                //ViewBag.id_usuari_alta = new SelectList(db.TBL_Usuaris, "id_usuari", tbl_temes.id_usuari_alta);
                return View(tbl_temes);
            }
            catch (Exception ex)
            {
                ViewBag.error = ex.Message;
                return View(tbl_temes);
            }
        }
        
        //
        // GET: /Tema/Edit/5
 
        public ActionResult Edit(string id)
        {
            try
            {
                TBL_Temes tbl_temes = db.TBL_Temes.Single(t => t.id_tema == id);
                return View(tbl_temes);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /Tema/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_Temes tbl_temes)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Temes.Attach(tbl_temes);
                    db.ObjectStateManager.ChangeObjectState(tbl_temes, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                return View(tbl_temes);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View(tbl_temes);
            }
        }

        //
        // GET: /Tema/Delete/5
 
        public ActionResult Delete(string id)
        {
            try
            {
                TBL_Temes tbl_temes = db.TBL_Temes.Single(t => t.id_tema == id);
                return View(tbl_temes);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /Tema/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(string id)
        {
            try
            {

                //canviem el codi ja que no esborrem sinò que editem els camps data de baixa 
                //i usuari de baixa 
                TBL_Temes tbl_temes = db.TBL_Temes.Single(t => t.id_tema == id);
                tbl_temes.id_usuari_baixa = Session["usuari"].ToString();//provisonal
                tbl_temes.data_baixa = System.DateTime.Now;
                //db.TBL_Temes.Attach(tbl_temes);
                db.ObjectStateManager.ChangeObjectState(tbl_temes, EntityState.Modified);
                db.SaveChanges();

                //donem de baixa els objectes lligats amb el tema
                UpdateFK(tbl_temes.id_tema, Session["usuari"].ToString());

                return RedirectToAction("Index");

            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// Exportem a excel
        /// </summary>
        /// <returns></returns>
        public ActionResult toExcel()
        {
            try
            {

                exportToExcel();
                return View();
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// métode per a exportar a Excel
        /// </summary>
        public void exportToExcel()
        {
            //This one is pretty easy, consisting of three parts:

            // * Creating the GridView placeholder
            // * Binding the query results to the GridView placeholder
            // * Dumping the results back to the browser

            // This example queries all contacts that have not unsubscribed, but you can imagine how easy it is to extend.
            var temes = db.SP_IndexTema();
            var grid = new System.Web.UI.WebControls.GridView();

            grid.DataSource = from tems in temes

                              select new
                              {
                                  Id_sap = tems.id_tema,
                                  Tema = tems.desc_tema,
                                  Id_usuari_alta = tems.id_usuari_alta,
                                  Data_alta = tems.data_alta
                              };
            grid.DataBind();

            Response.ClearContent();
            Response.AddHeader("content-disposition", "attachment; filename=sonin.xls");
            Response.ContentType = "application/excel";
            StringWriter sw = new StringWriter();
            HtmlTextWriter htw = new HtmlTextWriter(sw);
            grid.RenderControl(htw);

            Response.Write(sw.ToString());
            Response.End();
        }

        /// <summary>
        /// Actualitzem les dates de baixa i el usuari de baixa
        /// dels objectes relacionats amb el tema
        /// </summary>
        /// <param name="idTema">id de la SAP</param>
        /// <param name="idUsuariBaixa">id del  usuari de baixa</param>
        private void UpdateFK(String idTema, String idUsuariBaixa)
        {

            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand();
            System.Data.SqlClient.SqlConnection cn = new System.Data.SqlClient.SqlConnection();

            //la cadena de connexió
            cn.ConnectionString = "Server = (local); Database = sonin; User ID = sonin; Password = sonin; Trusted_Connection = False";

            // si la connexió està tancada la obrim.
            if (cn.State == ConnectionState.Closed)
            {
                cn.Open();
            }

            cmd.Connection = cn;
            //els fòrums
            cmd.CommandText = "UPDATE TBL_Forums SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();

            //els forums discussions
            cmd.CommandText = "UPDATE TBL_Forums_discussions SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();

            //els forums missatges
            cmd.CommandText = "UPDATE TBL_Forums_missatges SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();


            //les preguntes
            cmd.CommandText = "UPDATE TBL_Preguntes SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();

            //els valors recomanats
            cmd.CommandText = "UPDATE TBL_Valors_recomanats SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();

            //els valors possibles respostes
            cmd.CommandText = "UPDATE TBL_Valors_possibles_respostes SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();

            //els valors recomanats possibles respostes
            cmd.CommandText = "UPDATE TBL_Valors_recomanats_possibles_respostes SET data_baixa = '" + System.DateTime.Now.ToString() + "', SET id_usuari_baixa ='" + idUsuariBaixa + "'   WHERE id_tema = '" + idTema + "'";
            cmd.ExecuteNonQuery();


            //tanquem la connexió oberta
            if (cn.State == ConnectionState.Open)
            {
                cn.Close();
            }

        }


        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}