﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcSonin.DAL;
//per a exportar a excel
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

namespace MvcSonin.Controllers
{ 
    public class UPController : Controller
    {
        private SONINEntities db = new SONINEntities();

        //
        // GET: /UP/

        public ViewResult Index()
        {
            return View(db.TBL_UP.ToList());
        }

        //
        // GET: /UP/Details/5

        public ViewResult Details(string id)
        {
            try
            {
                //tractem la cadena id que rebem.
                int posSeparador = id.IndexOf("_");
                int longParam = id.Length;
                int diferencia = 0;
                diferencia = longParam - posSeparador;

                string idUP = id.Substring(0, posSeparador);
                string idSAP = id.Substring((posSeparador + 1), (diferencia - 1));

                TBL_UP tbl_up = db.TBL_UP.Single(t => t.codi_ics == idUP && t.id_sap == idSAP);
                return View(tbl_up);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // GET: /UP/Create

        public ActionResult Create()
        {
            //passem els valors de les SAP per a la combo
            ViewBag.id_sap = new SelectList(db.TBL_Saps, "id_sap", "desc_sap");
            return View();
        } 

        //
        // POST: /UP/Create

        [HttpPost]
        public ActionResult Create(TBL_UP tbl_up)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_UP.AddObject(tbl_up);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                //afegim per a que surti el valor escollit als desplegables
                ViewBag.id_sap = new SelectList(db.TBL_Saps, "id_sap", "desc_sap", tbl_up.id_sap);
                return View(tbl_up);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View(tbl_up);
            }

        }
        
        //
        // GET: /UP/Edit/5
 
        public ActionResult Edit(string id)
        {
            try
            {
                //tractem la cadena id que rebem.
                int posSeparador = id.IndexOf("_");
                int longParam = id.Length;
                int diferencia = 0;
                diferencia = longParam - posSeparador;

                string idUP = id.Substring(0, posSeparador);
                string idSAP = id.Substring((posSeparador + 1), (diferencia - 1));

                TBL_UP tbl_up = db.TBL_UP.Single(t => t.codi_ics == idUP && t.id_sap == idSAP);
                return View(tbl_up);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /UP/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_UP tbl_up)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_UP.Attach(tbl_up);
                    db.ObjectStateManager.ChangeObjectState(tbl_up, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_up);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View(tbl_up);
            }

        }

        //
        // GET: /UP/Delete/5
 
        public ActionResult Delete(string id)
        {
            try
            {
                //tractem la cadena id que rebem.
                int posSeparador = id.IndexOf("_");
                int longParam = id.Length;
                int diferencia = 0;
                diferencia = longParam - posSeparador;

                string idUP = id.Substring(0, posSeparador);
                string idSAP = id.Substring((posSeparador + 1), (diferencia - 1));

                TBL_UP tbl_up = db.TBL_UP.Single(t => t.codi_ics == idUP && t.id_sap == idSAP);
                return View(tbl_up);
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        //
        // POST: /UP/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(string id)
        {
            //tractem la cadena id que rebem.
            int posSeparador = id.IndexOf("_");
            int longParam = id.Length;
            int diferencia = 0;
            diferencia = longParam - posSeparador;

            string idUP = id.Substring(0, posSeparador);
            string idSAP = id.Substring((posSeparador + 1), (diferencia - 1));

            TBL_UP tbl_up = db.TBL_UP.Single(t => t.codi_ics == idUP && t.id_sap == idSAP);

            //No esborrem el registre, editem la data de alta
            tbl_up.data_baixa = System.DateTime.Now;
            db.ObjectStateManager.ChangeObjectState(tbl_up, EntityState.Modified);
            db.SaveChanges();

            //donem de baixa els objectes lligats amb la UP
            UpdateFK(tbl_up.id_sap, Session["usuari"].ToString());
            return RedirectToAction("Index");
        }

        /// <summary>
        /// Exportem a excel
        /// </summary>
        /// <returns></returns>
        public ActionResult toExcel()
        {
            try
            {

                exportToExcel();
                return View();
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// métode per a exportar a Excel
        /// </summary>
        public void exportToExcel()
        {
           //This one is pretty easy, consisting of three parts:

           // * Creating the GridView placeholder
           // * Binding the query results to the GridView placeholder
           // * Dumping the results back to the browser

           // This example queries all contacts that have not unsubscribed, but you can imagine how easy it is to extend.
            var upes = db.TBL_UP.ToList();
            var grid = new System.Web.UI.WebControls.GridView();

            grid.DataSource = from ups in upes
           
            select new
            {
                Id_sap= ups.id_sap,
                Codi_up = ups.codi_ics,
                Nom_up = ups.desc_up,
                Data_alta = ups.data_alta
            };
            grid.DataBind();

            Response.ClearContent();
            Response.AddHeader("content-disposition", "attachment; filename=sonin.xls");
            Response.ContentType = "application/excel";
            StringWriter sw = new StringWriter();
            HtmlTextWriter htw = new HtmlTextWriter(sw);
            grid.RenderControl(htw);
            Response.Write(sw.ToString());
            Response.End();
        }


        /// <summary>
        /// Actualitzem les dates de baixa i el usuari de baixa
        /// dels objectes relacionats amb la UP
        /// </summary>
        /// <param name="idUP">id de la UP</param>
        /// <param name="idUsuariBaixa">id del  usuari de baixa</param>
        private void UpdateFK(String idUP, String idUsuariBaixa)
        {

            System.Data.SqlClient.SqlCommand cmd = new System.Data.SqlClient.SqlCommand();
            System.Data.SqlClient.SqlConnection cn = new System.Data.SqlClient.SqlConnection();

            //la cadena de connexió
            cn.ConnectionString = "Server = (local); Database = sonin; User ID = sonin; Password = sonin; Trusted_Connection = False";

            // si la connexió està tancada la obrim.
            if (cn.State == ConnectionState.Closed)
            {
                cn.Open();
            }

            cmd.Connection = cn;
     
            //els centres
            cmd.CommandText = "UPDATE TBL_Centres SET data_baixa = '" + System.DateTime.Now.ToString() + "' WHERE codi_ics = '" + idUP + "'";
            cmd.ExecuteNonQuery();

            //tanquem la connexió oberta
            if (cn.State == ConnectionState.Open)
            {
                cn.Close();
            }

        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}