﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Entity;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using MvcSonin.DAL;
//per a exportar a excel
using System.Web.UI;
using System.Web.UI.WebControls;
using System.IO;

namespace MvcSonin.Controllers
{ 
    public class UsuariController : Controller
    {
        private SONINEntities db = new SONINEntities();

        //
        // GET: /Usuari/

        public ViewResult Index()
        {
            return View(db.TBL_Usuaris.ToList());
        }

        //
        // GET: /Usuari/Details/5

        public ViewResult Details(string id)
        {
            try
            {
                TBL_Usuaris tbl_usuaris = db.TBL_Usuaris.Single(t => t.id_usuari == id.ToLower());
                return View(tbl_usuaris);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // GET: /Usuari/Create

        public ActionResult Create()
        {
            return View();
        } 

        //
        // POST: /Usuari/Create

        [HttpPost]
        public ActionResult Create(TBL_Usuaris tbl_usuaris)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Usuaris.AddObject(tbl_usuaris);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }

                return View(tbl_usuaris);

            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }
        
        //
        // GET: /Usuari/Edit/5
 
        public ActionResult Edit(string id)
        {
            try
            {
                TBL_Usuaris tbl_usuaris = db.TBL_Usuaris.Single(t => t.id_usuari == id.ToLower());
                return View(tbl_usuaris);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // POST: /Usuari/Edit/5

        [HttpPost]
        public ActionResult Edit(TBL_Usuaris tbl_usuaris)
        {
            try
            {
                if (ModelState.IsValid)
                {
                    db.TBL_Usuaris.Attach(tbl_usuaris);
                    db.ObjectStateManager.ChangeObjectState(tbl_usuaris, EntityState.Modified);
                    db.SaveChanges();
                    return RedirectToAction("Index");
                }
                return View(tbl_usuaris);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // GET: /Usuari/Delete/5
 
        public ActionResult Delete(string id)
        {
            try
            {
                TBL_Usuaris tbl_usuaris = db.TBL_Usuaris.Single(t => t.id_usuari == id.ToLower());
                return View(tbl_usuaris);
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }

        //
        // POST: /Usuari/Delete/5

        [HttpPost, ActionName("Delete")]
        public ActionResult DeleteConfirmed(string id)
        {
            try
            {
                TBL_Usuaris tbl_usuaris = db.TBL_Usuaris.Single(t => t.id_usuari == id.ToLower());
                db.TBL_Usuaris.DeleteObject(tbl_usuaris);
                db.SaveChanges();
                return RedirectToAction("Index");
            }
            catch (Exception ex)
            {
                ViewBag.MissatgeError = ex.Message;
                return View("Error");
            }
        }


        /// <summary>
        /// Exportem a excel
        /// </summary>
        /// <returns></returns>
        public ActionResult toExcel()
        {
            try
            {

                exportToExcel();
                return View();
            }
            catch (Exception e)
            {
                ViewBag.error = e.Message;
                return View();
            }
        }

        /// <summary>
        /// métode per a exportar a Excel
        /// </summary>
        public void exportToExcel()
        {
            //This one is pretty easy, consisting of three parts:

            // * Creating the GridView placeholder
            // * Binding the query results to the GridView placeholder
            // * Dumping the results back to the browser

            // This example queries all contacts that have not unsubscribed, but you can imagine how easy it is to extend.
            var datos = db.TBL_Usuaris.ToList();
            var grid = new System.Web.UI.WebControls.GridView();

            grid.DataSource = from container in datos

                              select new
                              {
                                  Id_usuari = container.id_usuari,
                                  Nom_usuari = container.nom_usuari,
                                  Cognom_1 = container.cg1_usuari,
                                  Cognom_2 = container.cg2_usuari,
                                  Codi_sap = container.codi_sap,
                                  Codi_up= container.codi_ics,
                                  Mail = container.mail,
                                  Telefon = container.telefon,
                                  Id_usuari_alta = container.id_usuari_alta,
                                  Data_alta = container.data_alta
                              };
            grid.DataBind();

            Response.ClearContent();
            Response.AddHeader("content-disposition", "attachment; filename=sonin.xls");
            Response.ContentType = "application/excel";
            StringWriter sw = new StringWriter();
            HtmlTextWriter htw = new HtmlTextWriter(sw);
            grid.RenderControl(htw);
            Response.Write(sw.ToString());
            Response.End();
        }
        // Cridem la vista de error
        // GET: /Usuari/Error

        public ViewResult Error()
        {
            return View();
        }

        protected override void Dispose(bool disposing)
        {
            db.Dispose();
            base.Dispose(disposing);
        }
    }
}